#include <p33exxxx.h>

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C2.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 60000000L

int main( void )
{
	unsigned long i = 0;

	// FREF = Crystal / 2 = 12M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 2 = 240M Hz / 2 = 120M Hz
	CLKDIVbits.PLLPOST = 0;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	while( 1 )
	{	
		LATAbits.LATA4 = !LATAbits.LATA4;
		LATAbits.LATA5 = !LATAbits.LATA5;
		LATAbits.LATA6 = !LATAbits.LATA6;
		LATAbits.LATA7 = !LATAbits.LATA7;

		for( i = 0 ; i < 600000L ; i++ );
	}	

	while( 1 );
}

