#include <p33exxxx.h>
#include <timer.h>

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C2.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 60000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	LATAbits.LATA4 = !LATAbits.LATA4;
	LATAbits.LATA5 = !LATAbits.LATA5;
	LATAbits.LATA6 = !LATAbits.LATA6;
	LATAbits.LATA7 = !LATAbits.LATA7;
}

int main( void )
{
	// FREF = Crystal / 2 = 12M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 2 = 240M Hz / 2 = 120M Hz
	CLKDIVbits.PLLPOST = 0;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	while( 1 );
}

