#include <p33exxxx.h>
#include <timer.h>
#include <adc.h>
#include "..\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C2.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 30000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10

const unsigned char LCMString1[ ] = "dsPIC33EPxxMU810"; 
const unsigned char LCMString2[ ] = "VR:    ,    ,   ";
unsigned int VRValue = 0;
unsigned char Timer1TickFlag = 0;

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag++;

	LATAbits.LATA4 = !LATAbits.LATA4;
	LATAbits.LATA5 = !LATAbits.LATA5;
	LATAbits.LATA6 = !LATAbits.LATA6;
	LATAbits.LATA7 = !LATAbits.LATA7;
}

int main( void )
{
	// FREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 4 = 240M Hz / 4 = 60M Hz
	CLKDIVbits.PLLPOST = 1;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenADC1( ADC_MODULE_ON & ADC_IDLE_CONTINUE & ADC_ADDMABM_SCATTR & ADC_AD12B_10BIT & ADC_FORMAT_INTG &
			   ADC_SSRC_AUTO & ADC_AUTO_SAMPLING_ON & ADC_MULTIPLE & ADC_SAMP_ON ,
			  ADC_VREF_AVDD_AVSS & ADC_SCAN_ON & ADC_SELECT_CHAN_0 & ADC_SETS_OF_SAMPLES_PER_INT_3 &
			  ADC_ALT_BUF_OFF & ADC_ALT_INPUT_OFF ,
			  ADC_SAMPLE_TIME_31 & ADC_CONV_CLK_INTERNAL_RC & ADC_CONV_CLK_256Tcy ,
			  ADC_DMA_DIS & ADC_DMA_BUF_LOC_1 ,
			  0x00 , ENABLE_AN2_ANA | ENABLE_AN4_ANA | ENABLE_AN12_ANA , 0x00 , 0x00 ,
			  SCAN_NONE_16_31 ,
			  SKIP_SCAN_AN0 & SKIP_SCAN_AN1 & SKIP_SCAN_AN3 & SKIP_SCAN_AN5 & SKIP_SCAN_AN6 &
			  SKIP_SCAN_AN7 & SKIP_SCAN_AN8 & SKIP_SCAN_AN9 & SKIP_SCAN_AN10 &
			  SKIP_SCAN_AN11 & SKIP_SCAN_AN13 & SKIP_SCAN_AN14 &
			  SKIP_SCAN_AN15 );

	while( 1 )
	{ 
		if( Timer1TickFlag ) 	
		{

			VRValue = ( unsigned int ) ReadADC1( 2 );
			LCM_SetCursor( 3 , 1 );
			LCM_PutNumber( VRValue , 4 );

			VRValue = ( unsigned int ) ReadADC1( 0 );
			LCM_SetCursor( 8 , 1 );
			LCM_PutNumber( VRValue  , 4 );

			VRValue = ( unsigned int ) ReadADC1( 1 );
			LCM_SetCursor( 13 , 1 );
			LCM_PutNumber( VRValue >> 1 , 3 );
		}
	}

	while( 1 );
}
