#include <p33exxxx.h>
#include <timer.h>
#include <pps.h>
#include <uart.h>
#include <stdio.h>
#include "..\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C2.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 30000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10
#define UART1Period	( ( SystemFrequency / UART1Baud ) / 16 ) - 1
#define UART1Baud 19200L

const unsigned char LCMString1[ ] = "dsPIC33EPxxMU810"; 
const unsigned char LCMString2[ ] = "Rx:";

unsigned char UARTString1[ 32 ]; 
unsigned char UART_IsReceived = 0;
unsigned char UART_ReceiveBuffer = 0;
unsigned char Timer1TickFlag = 0;
unsigned char Hours = 0 , Mins = 0 , Secs = 0;

void UART_PutRAMString( unsigned char * String );

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	static unsigned char i = 0;

	IFS0bits.T1IF = 0;

	if( i++ > 9 )
	{
		i = 0;

		Timer1TickFlag++;
	
		LATAbits.LATA4 = !LATAbits.LATA4;
		LATAbits.LATA5 = !LATAbits.LATA5;
		LATAbits.LATA6 = !LATAbits.LATA6;
		LATAbits.LATA7 = !LATAbits.LATA7;
	}
}

void __attribute__( ( interrupt , no_auto_psv ) ) _U1RXInterrupt( void )
{
	IFS0bits.U1RXIF = 0;

	UART_ReceiveBuffer = ReadUART1( );

	UART_IsReceived = 1;
}

int main( void )
{
	unsigned char Cursor = 3;

	// FREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 4 = 240M Hz / 4 = 60M Hz
	CLKDIVbits.PLLPOST = 1;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	PPSUnLock;								// __builtin Function
	iPPSInput( IN_FN_PPS_U1RX , IN_PIN_PPS_RP98 );
	iPPSOutput( OUT_PIN_PPS_RP104 , OUT_FN_PPS_U1TX );
	PPSLock;								// __builtin Function

	OpenUART1( UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE & UART_MODE_SIMPLEX & UART_UEN_00 & 
			   UART_EN_WAKE & UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_UXRX_IDLE_ONE & 
			   UART_NO_PAR_8BIT & UART_BRGH_SIXTEEN & UART_1STOPBIT ,
			   UART_INT_TX & UART_IrDA_POL_INV_ZERO & UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE &
			   UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR ,
			   UART1Period );

	ConfigIntUART1( UART_RX_INT_EN & UART_RX_INT_PR4 & UART_TX_INT_DIS & UART_TX_INT_PR4 ); 

	while( 1 )
	{
		// UART ?
		if( UART_IsReceived )
		{
			UART_IsReceived = 0;

			// ]wCСAXrC
			LCM_SetCursor( Cursor++ , 1 );
			LCM_PutASCII( UART_ReceiveBuffer );

			if( Cursor > 15 ) Cursor = 3;
		}

		// BzC
		if( Timer1TickFlag ) 	
		{
			if( ++Secs > 59 ) 
			{
				Secs = 0;
				if( ++Mins > 59 )
				{
					Mins = 0;
					if( ++Hours > 23 )
						Hours = 0;
				}
			}

			sprintf( ( char * ) UARTString1 , "Now Time %2d:%2d:%2d\r" ,
					 Hours , Mins , Secs );

			UART_PutRAMString( UARTString1 );

			Timer1TickFlag--;
		}
	}

	while( 1 );
}

void UART_PutRAMString( unsigned char * String ) 
{
	while( *String != 0x00 )
	{
		while( BusyUART1( ) );
	    WriteUART1( *String++ );
	}
}

