#include <p33exxxx.h>
#include <timer.h>
#include <i2c.h>
#include "..\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C1.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 30000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10
#define UART1Period	( ( SystemFrequency / UART1Baud ) / 16 ) - 1
#define UART1Baud 19200L
#define I2C1Baud ( ( SystemFrequency / I2C1BaudRate ) - 1 )
#define I2C1BaudRate 10000L

const unsigned char LCMString1[ ] = "dsPIC33EPxxMU810"; 
const unsigned char LCMString2[ ] = "Write:  ,Read:  "; 
unsigned char Timer1TickFlag = 0;

void I2C_EE10BitsByteWirte( unsigned char SalveAddr , unsigned int Addr , unsigned char Data );
unsigned char I2C_EE10BitsRandomRead( unsigned char SalveAddr , unsigned int Addr );

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag++;

	LATAbits.LATA4 = !LATAbits.LATA4;
	LATAbits.LATA5 = !LATAbits.LATA5;
	LATAbits.LATA6 = !LATAbits.LATA6;
	LATAbits.LATA7 = !LATAbits.LATA7;
}

int main( void )
{
	unsigned long i = 0;
	unsigned int I2CEEAddr = 0;

	// FREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 4 = 240M Hz / 4 = 60M Hz
	CLKDIVbits.PLLPOST = 1;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	OpenI2C1( I2C1_ON & I2C1_IDLE_CON & I2C1_CLK_REL & I2C1_IPMI_DIS & I2C1_7BIT_ADD &
			  I2C1_SLW_DIS & I2C1_SM_DIS & I2C1_GCALL_DIS & I2C1_STR_DIS & I2C1_NACK &
			  I2C1_ACK_DIS & I2C1_RCV_DIS & I2C1_STOP_DIS & I2C1_RESTART_DIS & I2C1_START_DIS ,
			  I2C1Baud );

	while( 1 )
	{
		if( Timer1TickFlag > 0 )
 		{
			Timer1TickFlag--;

			LCM_SetCursor( 6 , 1 );
			LCM_PutHex( I2CEEAddr );
			I2C_EE10BitsByteWirte( 0xA0 , I2CEEAddr, I2CEEAddr ); 
		
			// ! EEPROMgJ, ݭn5mS, pGis, |_gJu@,
			// BH²檺n驵Bz, ѩ󤣦PsĶ,
			// n驵Ҳͪɶj@, SO`N!
			// Ϊ̧ĥεw骺pɨӳBz.
			for( i = 0; i < 10000 ; i++ );
			 
			LCM_SetCursor( 14 , 1 );
			LCM_PutHex( I2C_EE10BitsRandomRead( 0xA0 , I2CEEAddr ) );

			if( ++I2CEEAddr > 0xfff )
				I2CEEAddr = 0;
		}
	}

	while( 1 );
}

void I2C_EE10BitsByteWirte( unsigned char SalveAddr , unsigned int Addr , unsigned char Data )
{
	IdleI2C1( );

	StartI2C1( );
	while( I2C1CONbits.SEN );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( SalveAddr & 0xfffe );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( ( Addr >> 8 ) & 0xff );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( Addr & 0xff );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( Data & 0xff );

	StopI2C1( ) ;
	while ( I2C1CONbits.PEN ) ;	
}

unsigned char I2C_EE10BitsRandomRead( unsigned char SalveAddr , unsigned int Addr )
{
	char Data = 0;

	IdleI2C1( );

	StartI2C1( );
	while( I2C1CONbits.SEN );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( SalveAddr & 0xfffe );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( ( Addr >> 8 ) & 0xff );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( Addr & 0xff );

	RestartI2C1( );
	while( I2C1CONbits.RSEN );

	// Ready to Transmit ?
	while( I2C1STATbits.TBF );
	MasterWriteI2C1( SalveAddr | 0x0001 );

	Data = MasterReadI2C1( );

	StopI2C1( );
	while ( I2C1CONbits.PEN ) ;	

	return Data;
}
