#include <p33exxxx.h>
#include <timer.h>
#include <pps.h>
#include <spi.h>
#include "..\App1632_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C1.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 30000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10

#define DAC_CS LATDbits.LATD2
#define DAC_CS_TRIS TRISDbits.TRISD2
#define DAC_LDAC LATDbits.LATD1
#define DAC_LDAC_TRIS TRISDbits.TRISD1

const unsigned char LCMString1[ ] = "dsPIC33EPxxMU810"; 
const unsigned char LCMString2[ ] = "ChA:    ChB:    "; 
unsigned char Timer1TickFlag = 0;

void SPI_DACUpdate( unsigned int ChAValue , unsigned int ChBValue );

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag++;

	LATAbits.LATA4 = !LATAbits.LATA4;
	LATAbits.LATA5 = !LATAbits.LATA5;
	LATAbits.LATA6 = !LATAbits.LATA6;
	LATAbits.LATA7 = !LATAbits.LATA7;
}

int main( void )
{
	unsigned int DACChAValue = 0 , DACChBValue = 0;

	// FREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;

	// FOSC = FVOC / 4 = 240M Hz / 4 = 60M Hz
	CLKDIVbits.PLLPOST = 1;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );

	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );

	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	ANSELAbits.ANSA6 = 1;
	ANSELAbits.ANSA7 = 1;

	TRISAbits.TRISA4 = 0;
	TRISAbits.TRISA5 = 0;
	TRISAbits.TRISA6 = 0;
	TRISAbits.TRISA7 = 0;
	
	LATAbits.LATA4 = 1;
	LATAbits.LATA5 = 0;
	LATAbits.LATA6 = 1;
	LATAbits.LATA7 = 0;

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	PPSUnLock;								// __builtin Function
	iPPSInput( IN_FN_PPS_SDI1 , IN_PIN_PPS_RP124 );
	iPPSOutput( OUT_PIN_PPS_RP112 , OUT_FN_PPS_SCK1 );
	iPPSOutput( OUT_PIN_PPS_RP113 , OUT_FN_PPS_SDO1 );
	PPSLock;								// __builtin Function

	DAC_CS_TRIS = 0;
	DAC_CS = 1;
	DAC_LDAC_TRIS = 0;
	DAC_LDAC = 1;

	OpenSPI1( ENABLE_SCK_PIN & ENABLE_SDO_PIN & SPI_MODE16_OFF & SPI_SMP_OFF & SPI_CKE_ON & SLAVE_ENABLE_OFF &
			  CLK_POL_ACTIVE_HIGH & MASTER_ENABLE_ON & SEC_PRESCAL_8_1 & PRI_PRESCAL_64_1 ,
			  FRAME_ENABLE_OFF & FRAME_SYNC_OUTPUT & FRAME_POL_ACTIVE_LOW & 
			  FRAME_SYNC_EDGE_PRECEDE & FIFO_BUFFER_DISABLE ,
			  SPI_ENABLE & SPI_IDLE_CON & SPI_RX_OVFLOW_CLR & 
			  BUF_ELE_COUNT_0 & BUF_INT_SEL_0 & SPI_SHFTREG_NOT_EMPTY & RX_FIFO_NOT_EMPTY );

	while( 1 )
	{
		if( Timer1TickFlag > 0 )
		{
			Timer1TickFlag--;
	
			SPI_DACUpdate( DACChAValue , DACChBValue );

			LCM_SetCursor( 4 , 1 );
			LCM_PutNumber( DACChAValue , 4 );
			LCM_SetCursor( 12 , 1 );
			LCM_PutNumber( DACChBValue , 4 );
	
			if( ( DACChAValue += 40 ) > 2399 )
				DACChAValue = 0;

			if( ( DACChBValue += 40 ) > 4095 )
				DACChBValue = 0;
		}
	}

	while( 1 );
}

void SPI_DACUpdate( unsigned int ChAValue , unsigned int ChBValue )
{
	unsigned long i = 0;

	// CS Active
	DAC_CS = 0;

	while( SPI1STATbits.SPITBF );
	WriteSPI1( 0x70 | ( ( ChAValue >> 8 ) & 0x000f ) );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	while( SPI1STATbits.SPITBF );
	WriteSPI1( ChAValue & 0x00ff );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_CS = 1;

	// CS Active
	DAC_CS = 0;

	while( SPI1STATbits.SPITBF );
	WriteSPI1( 0xf0 | ( ( ChBValue >> 8 ) & 0x000f ) );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	while( SPI1STATbits.SPITBF );
	WriteSPI1( ChBValue& 0x00ff );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_CS = 1;

	// CS Active
	DAC_LDAC = 0;
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_LDAC = 1;
}
